@@ -8,7 +8,66 @@ from pymarc.field import Field, SUBFIELD_INDICATOR, END_OF_FIELD, \
         map_marc8_field
 from pymarc.marc8 import marc8_to_unicode
 
-isbn_regex = re.compile(r'([0-9\-]+)')
+try:
+    # the json module was included in the stdlib in python 2.6
+    # http://docs.python.org/library/json.html
+    import json
+except ImportError:
+    # simplejson 2.0.9 is available for python 2.4+
+    # http://pypi.python.org/pypi/simplejson/2.0.9
+    # simplejson 1.7.3 is available for python 2.3+
+    # http://pypi.python.org/pypi/simplejson/1.7.3
+    import simplejson as json
+
+try:
+    # izip_longest first appeared in python 2.6
+    # http://docs.python.org/library/itertools.html#itertools.izip_longest
+    from itertools import izip_longest
+except ImportError:
+    # itertools was introducted in python 2.3
+    # we just define the required classes and functions
+    # for 2.3 <= python < 2.6 here
+    class ZipExhausted(Exception):
+        pass
+
+    def _next(obj):
+        """
+        ``next`` (http://docs.python.org/library/functions.html#next)
+        was introduced in python 2.6 - and if we are here
+        (no ``izip_longest``), than we need to define this."""
+        return obj.next()
+
+    def izip_longest(*args, **kwds):
+        """
+        Make an iterator that aggregates elements from each of the iterables.
+        If the iterables are of uneven length, missing values are filled-in
+        with fillvalue.
+        Iteration continues until the longest iterable is exhausted.
+
+        This function is available in the standard lib since 2.6.
+        """
+        # chain and repeat are available since python 2.3
+        from itertools import chain, repeat
+
+        # izip_longest('ABCD', 'xy', fillvalue='-') --> Ax By C- D-
+        fillvalue = kwds.get('fillvalue', '')
+        counter = [len(args) - 1]
+        def sentinel():
+            if not counter[0]:
+                raise ZipExhausted
+            counter[0] -= 1
+            yield fillvalue
+        fillers = repeat(fillvalue)
+        iterators = [chain(it, sentinel(), fillers) for it in args]
+        try:
+            while iterators:
+                yield tuple(map(_next, iterators))
+        except ZipExhausted:
+            pass
+        finally:
+            del chain
+
+isbn_regex = re.compile(r'([0-9\-xX]+)')
 
 class Record(object):
     """
@@ -96,6 +155,56 @@ class Record(object):
         """
         self.fields.extend(fields)
 
+    def add_grouped_field(self, *fields):
+        """
+        add_grouped_field() will add pymarc.Field objects to a Record object,
+        attempting to maintain a loose numeric order per the MARC standard for
+        "Organization of the record" (http://www.loc.gov/marc/96principl.html)
+        Optionally you can pass in multiple fields.
+        """
+        for f in fields:
+            if len(self.fields) == 0 or not f.tag.isdigit():
+                self.fields.append(f)
+                continue 
+            self._sort_fields(f, 'grouped')
+
+    def add_ordered_field(self, *fields):
+        """
+        add_ordered_field() will add pymarc.Field objects to a Record object,
+        attempting to maintain a strict numeric order.
+        Optionally you can pass in multiple fields.
+        """
+        for f in fields:
+            if len(self.fields) == 0 or not f.tag.isdigit():
+                self.fields.append(f)
+                continue 
+            self._sort_fields(f, 'ordered')
+
+    def _sort_fields(self, field, mode):
+        if mode == 'grouped':
+            tag = int(field.tag[0])
+        else:
+            tag = int(field.tag)
+
+        i, last_tag = 0, 0
+        for selff in self.fields:
+            i += 1
+            if not selff.tag.isdigit():
+                self.fields.insert(i - 1, field)
+                break
+
+            if mode == 'grouped':
+                last_tag = int(selff.tag[0])
+            else:
+                last_tag = int(selff.tag)
+
+            if last_tag > tag:
+                self.fields.insert(i - 1, field)
+                break
+            if len(self.fields) == i:
+                self.fields.append(field)
+                break
+
     def remove_field(self, *fields):
         """
         remove_field() will remove one or more pymarc.Field objects from
@@ -251,6 +360,29 @@ class Record(object):
     # alias for backwards compatability
     as_marc21 = as_marc
 
+    def as_dict(self):
+        """
+        Turn a MARC record into a dict, which is used for ``as_json``.
+        """
+        _dict = {}
+        _dict['leader'] = self.leader
+        _dict['fields'] = {}
+        for field in self:
+            if hasattr(field, 'subfields'):
+                _dict['fields'][field.tag] = {}
+                _dict['fields'][field.tag]['indicators'] = field.indicators
+                _dict['fields'][field.tag]['subfields'] = dict(
+                    izip_longest(*[iter(field.subfields)] * 2))
+            else:
+                _dict['fields'][field.tag] = field.data
+        return _dict
+
+    def as_json(self, **kwargs):
+        """
+        Serialize a record as JSON.
+        """
+        return json.dumps(self.as_dict(), **kwargs)
+
     def title(self):
         """
         Returns the title of the record (245 $a an $b).
@@ -269,7 +401,8 @@ class Record(object):
     def isbn(self):
         """
         Returns the first ISBN in the record or None if one is not
-        present. The returned ISBN will be all numberic; so dashes and 
+        present. The returned ISBN will be all numeric, except for an
+        x/X which may occur in the checksum position.  Dashes and 
         extraneous information will be automatically removed. If you need 
         this information you'll want to look directly at the 020 field, 
         e.g. record['020']['a']
